/* $Id: automat.h,v 1.9 2002/06/14 13:54:04 aa5779 Exp $ */
/*
	Copyright (C) 2001 Artem V. Andreev (artem@AA5779.spb.edu)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#ifndef AUTOMAT_H
#define AUTOMAT_H

#include <stdlib.h>
#include "lists.h"
#include "shared.h"

DEFLIST(range, 
		int start;
		int end;
		int repeat;
		int dither;
		);

enum node_types { node_const, node_variable, node_assign,
					node_binop, node_unop, node_selection, node_rangecheck,
					node_autoseq, node_autoseqvar, node_autoseqstate
				};

enum node_variable {nv_input, nv_depth, nv_state, nv_topstack,
					nv_arg0 = nv_state, nv_arg1,
					nv_arg2, nv_arg3, nv_arg4, nv_arg5, nv_arg6, 
					nv_arg7, nv_arg8, nv_arg9,
					nv_multiplexor, 
					nv_last = nv_multiplexor
				   };
				
typedef int reg_block_t[nv_last + 1];
				   
DEFLIST(macrodef, 
		char *name;
		struct node *val;
		);

DEFLIST(rangedef,
		char *name;
		range *r;
		);
				
typedef int (*binop_t)(int, int);

typedef struct node
{
	int type;
	int ref_cnt;
	union
	{
		int val;
		struct
		{
			binop_t opf;
			struct node *left;
			struct node *right;
		} op;
		struct
		{
			range *r;
			struct node *right;
		} sel;
		struct
		{
			int regno;
			struct node *left;
		} a;
		struct
		{
			struct autoseq *seq;
			struct node *right;
			unsigned char reset : 1;
		} aseq;
		struct
		{
			struct autoseq *seq;
			int var;
		} ac;
	} x;
} node;

struct autoseqdef;

int select_value(range *r, int value);
int eval_node(node *n, reg_block_t args);

range *parse_range(char *start, char **end, rangedef **rgs);
int in_range(range *r, int value);
node *parse_expr(char *start, char **end, macrodef *macros, rangedef **rgs, 
				 struct autoseqdef *aseqs);

int eval_expr(char *start, char **end, macrodef *macros, rangedef *rgs, struct autoseqdef *aseqs);

enum automaton_results { ar_rejected, ar_accepted };

DEFLIST(transition,
		int state;
		node *filter;
		node *stackfilter;
		int newstate;
		int eat_input;
		node **tostack;
		node **outchars;
		);

DEFLIST(automaton, 
	char *name;
	macrodef *names;
	rangedef *ranges;
	struct autoseqdef *aseqs;
	transition *table;
	int start;
	int fin_cnt;
	int *finals;
);

automaton *create_automaton(char *name);
automaton *load_automaton(char *name);

automaton *get_au_byname(char *name);
void parse_automaton(automaton *au, char *start, char **end);


enum out_func { aof_normal = 0, aof_state, aof_topstack, aof_depth };

typedef struct auto_state
{
	automaton *au;
	int current;
	int depth;
	int maxdepth;
	int *stack;
	// private
	transition *active;
	node **active_out;
	reg_block_t regs;
} auto_state;

DEFLIST(autoseq,
		auto_state state;
		int inch;
		int start;
		unsigned char auto_reset : 1;
		unsigned char master : 1;
		int outfunc;
		int cur_multiplexor;
		int multiplexor;
		unsigned int mp_mask;
		);

DEFLIST(autoseqdef,
		shared_object sh;
		autoseq *seq;
	   );

#define AU_PATH_ENV "AU_PATH"
#define AU_START_STACK_DEPTH 64

int is_in_final(auto_state *as);

void autoseq_init(autoseq *seq);
autoseq *parse_autoseq(char *start);
int autoseq_step(autoseq *seq, int *inchar, int *outchar);
int need_feeding(autoseq *seq);
int as_get_reg(autoseq *seq, int reg);
int as_map_name(autoseq *seq, char *name);
char *as_getname_byid(autoseq *seq, int id, char *prefix);
int as_is_in_final(autoseq *seq);
void remove_autoseq(autoseq *as);

#endif // AUTOMAT_H
