/*
	Copyright (C) 2001 Artem V. Andreev (artem@AA5779.spb.edu)
        
        this file
        Copyright (C) 2001 Matt Boeh (mboeh@desperance.net)
        
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#include <modules.h>
#include <util.h>
#include <dstring.h>
#ifdef HAVE_SYS_TIMEB
        #include <sys/timeb.h>
#endif
#include <time.h>
#include <string.h>
#include "version.h"

static module *this_mod;
extern procstack *get_pstack(void);

void time_LTX_init(module *this)
{
	this_mod = this;
}

/* bi_time * timestamp() -
 * direct implementation of time() call. Returns seconds since the
 * Epoch. */
static void bi_time(chunk *current, interpreter *i, value *args)
{
        /* Who knows what kind of nutty long ints may be returned 
         * by time() in the future? Best prepare for the long run. */
        char time_buf[40];
        /* Convert the time from time_t to string. */
        sprintf(time_buf, "%u", (unsigned)time(NULL));
        /* Return the string. */
        ds_appendstr(get_pstack()->state->buffer, time_buf);
}

/* bi_timestr * ascii-time(time) -
 * implementation of ctime() library function. Takes an argument, time
 * in seconds since the Epoch (time() returns the current time handily)
 * The time returned is in a form like
 * "Wed Jun 30 21:49:08 1993" */
static void bi_timestr(chunk *current, interpreter *i, value *args)
{
        /* Iteration pointer */
        char *n;
        /* Temporary time variable */
        time_t ti;
        /* I hope no one has times longer than this */
        char time_buf[40];
        /* Convert the string arg to an integer and pass it to
         * ctime(). time_buf now holds the time with a newline. 
         * The string needs to be copied like so because ctime
         * returns a pointer to a static string which may change
         * at the use of any time function - not a good thing. */
        ti = (time_t)atoi(DS_BODY(args->val));
        n = ctime(&ti);
        strcpy(time_buf, n);
        /* Make n point to the beginning of time_buf */
        n = time_buf;
        /* Remove that pesky newline. */
        time_buf[strlen(time_buf)-1] = '\0';
        /* And return the string. */
        ds_appendstr(get_pstack()->state->buffer, time_buf);
}

#ifdef HAVE_STRFTIME
static void bi_strftime(chunk *current, interpreter *i, value *args)
{
	time_t src = convert_to_int(DS_BODY(args->val));
	char buffer[256];
	struct tm broken_down = *localtime(&src);
	strftime(buffer, sizeof(buffer), DS_BODY(next_value(args)->val), &broken_down);
	ds_appendstr(get_pstack()->state->buffer, buffer);
}
#endif

#ifdef HAVE_SLEEP
static void bi_sleep(chunk *current, interpreter *i, value *args)
{
	int slp = convert_to_int(DS_BODY(args->val));
	if(slp < 0)
		recover_error("invalid sleep argument: %d", slp);
	sleep(slp);
}
#endif

static master_instruction_src time_src[] = {
	 {NULL}
};

static master_instruction_src time_src_add[] = {
        {"timestamp", 1, bi_time, 0, 0, 0},
     	{"ascii-time", 1, bi_timestr, 1, 1, 1},
#ifdef HAVE_STRFTIME		
		{"strftime", 1, bi_strftime, 2, 2, 2},
#endif
#ifdef HAVE_SLEEP
		{"sleep", 0, bi_sleep, 1, 1, 1},
#endif
		{NULL}
};                                                                         

void time_LTX_attach(mod_attach *ma)
{
	ma->data = xmalloc(sizeof(time_src) * sizeof(instruction_t) / sizeof(*time_src));
	replace_builtins(ma->i, time_src, ma->data);
	append_builtins(ma->i, time_src_add);
}

const char time_LTX_version[] = VERSION;
const char time_LTX_date[] = __DATE__;
